/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef AIRPORTSEARCHDIALOG_H_
#define AIRPORTSEARCHDIALOG_H_

#include <gtkmm/dialog.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/box.h>
#include <libglademm.h>
#include "Airport.h"
#include "DataStore.h"
#include "Config.h"
#include "OnScreenKeypad.h"
#include "Route.h"

class AirportSearchDialogModelColumns : public Gtk::TreeModelColumnRecord
{
public:
	AirportSearchDialogModelColumns();

	Gtk::TreeModelColumn<std::string> AirportIdent;
	Gtk::TreeModelColumn<std::string> AirportName;
	Gtk::TreeModelColumn<Airport *> AirportData;
};

class AirportSearchDialog
{
public:
	AirportSearchDialog(std::string *GladeFilename, DataStore *Data, Config *Cfg, Route *RouteData);
	virtual ~AirportSearchDialog();

	int Run();
private:
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Dialog *GtkDialog;
	Gtk::Entry *AirportEntry;
	Gtk::TreeView *AirportsTreeView;
	Gtk::Button *InformationButton;
	Gtk::Button *AddtoRouteButton;
	Gtk::Button *CloseButton;
	Gtk::VBox *OnScreenKeypadParent;
	OnScreenKeypad *Keypad;

	AirportSearchDialogModelColumns Columns;
	Glib::RefPtr<Gtk::ListStore> ListStore;

	void on_AirportEntry_changed();
	void on_InformationButton_clicked();
	void on_AddtoRouteButton_clicked();
	void on_CloseButton_clicked();

	std::string *GladeFilename;
	Config *Cfg;
	DataStore *Data;
	Route *RouteData;

	/* Something to store our matching items in */
	class MatchingItem
	{
	public:
		MatchingItem(Airport *AirP);
		Airport *GetAirport();
		bool Matchi(std::string str);
	private:
		Airport *AirP;
	};
	std::vector<MatchingItem> MatchingItems;
};

#endif /*AIRPORTSEARCHDIALOG_H_*/
