/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <glibmm/i18n.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "ConfigDialog.h"

ConfigDialog::ConfigDialog(std::string *GladeFilename, Config *Cfg)
{
	this->Cfg=Cfg;

	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "ConfigDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog(_("Error reading data file: ")+*GladeFilename+"\n"+_("Program cannot continue."),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
		return;
	}

	refXml->get_widget("ConfigDialog",GtkDialog);
	refXml->get_widget("SaveNow",SaveNow);
	refXml->get_widget("LoadNow",LoadNow);

	refXml->get_widget("gpsd",gpsd);
	refXml->get_widget("gpsdHost",gpsdHost);
	refXml->get_widget("gpsdPort",gpsdPort);
	refXml->get_widget("SerialPort",SerialPort);
	refXml->get_widget("Simulated",Simulated);
	refXml->get_widget("SerialPortDevice",SerialPortDevice);
	refXml->get_widget("SerialPortBaud",SerialPortBaud);
	refXml->get_widget("AutoActivateGPS",AutoActivateGPS);
	refXml->get_widget("AutoStartgpsd",AutoStartgpsd);
	refXml->get_widget("GPSAutoSelectChart",AutoSelectChart);
	refXml->get_widget("PDFViewer",PDFViewer);
	refXml->get_widget("ImageViewer",ImageViewer);
	refXml->get_widget("SaveConfigOnExit",SaveConfigOnExit);
	refXml->get_widget("UserDataDir",UserDataDir);
	refXml->get_widget("DataStoreDir",DataStoreDir);
	refXml->get_widget("METARSourceURL",METARSourceURL);
	refXml->get_widget("CenterChartOnGPS",CenterChartOnGPS);
	refXml->get_widget("gpsdCmd",gpsdCmd);
	refXml->get_widget("GPSUpdateFrequency",GPSUpdateFrequency);
	refXml->get_widget("NationalFlightDBImportDir",NationalFlightDBImportDir);
	refXml->get_widget("XPlane900DataImportDir",XPlane900DataImportDir);
	refXml->get_widget("SectionalsImportDir",SectionalsImportDir);
	refXml->get_widget("dTPPImportDir",dTPPImportDir);
	refXml->get_widget("DigitalAeroInfoImportDir",DigitalAeroInfoImportDir);
	refXml->get_widget("DAILabel",DAILabel);
	refXml->get_widget("SimLon",SimLon);
	refXml->get_widget("SimLat",SimLat);
	refXml->get_widget("SimTrk",SimTrk);
	refXml->get_widget("SimSpd",SimSpd);
	refXml->get_widget("SimAlt",SimAlt);
	refXml->get_widget("DisableSS",DisableSS);
	refXml->get_widget("DisablePM",DisablePM);
	refXml->get_widget("ShowWelcomeOnStartup",ShowWelcomeOnStartupCheckbox);

	SaveNow->signal_clicked().connect(sigc::mem_fun(this,&ConfigDialog::on_SaveNow_clicked));
	LoadNow->signal_clicked().connect(sigc::mem_fun(this,&ConfigDialog::on_LoadNow_clicked));

	DigitalAeroInfoImportDir->set_sensitive(false);
	DigitalAeroInfoImportDir->hide();
	DAILabel->hide();
}

ConfigDialog::~ConfigDialog() {
	delete GtkDialog;
}

int ConfigDialog::Run() {
	int response;

	GetConfig();
	response=GtkDialog->run();
	if (response==Gtk::RESPONSE_OK) {
		SetConfig();
	}
	return response;
}

void ConfigDialog::on_SaveNow_clicked() {
	SetConfig();
	if (!Cfg->Save()) {
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error saving configuration file.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		delete message;
	}
}

void ConfigDialog::on_LoadNow_clicked() {
	if (!Cfg->Load()) {
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error loading configuration file.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		delete message;
	} else {
		GetConfig();
	}
}

void ConfigDialog::SetConfig() {
	/* Not implemented yet
	Cfg->SetSerialPortDevice(SerialPortDevice->get_text());
	Cfg->SetSerialPortBaud(SerialPortBaud->???
	*/
	if (gpsd->get_active())
		Cfg->SetGPSSource(Config::gpsd);
	Cfg->SetgpsdHost(gpsdHost->get_text());
	Cfg->SetgpsdPort(gpsdPort->get_text());
	if (SerialPort->get_active())
		Cfg->SetGPSSource(Config::Serial);
	if (Simulated->get_active())
		Cfg->SetGPSSource(Config::Simulated);
	Cfg->SetAutoActivateGPS(AutoActivateGPS->get_active());
	Cfg->SetAutoStartgpsd(AutoStartgpsd->get_active());
	Cfg->SetAutoSelectChart(AutoSelectChart->get_active());
	Cfg->SetPDFViewer(PDFViewer->get_text());
	Cfg->SetImageViewer(ImageViewer->get_text());
	Cfg->SetSaveConfigOnExit(SaveConfigOnExit->get_active());
	Cfg->SetUserDataDir(UserDataDir->get_text());
	Cfg->SetDataStoreDir(DataStoreDir->get_text());
	Cfg->SetMETARSourceURL(METARSourceURL->get_text());
	Cfg->SetCenterOnGPS(CenterChartOnGPS->get_active());
	Cfg->SetgpsdCmd(gpsdCmd->get_text());
	Cfg->SetGPSUpdateFrequency((unsigned int)rint(GPSUpdateFrequency->get_value()*1000.0));
	Cfg->SetdTPPImportDir(dTPPImportDir->get_text());
	Cfg->SetSectionalsImportDir(SectionalsImportDir->get_text());
	Cfg->SetDigitalAeroInfoImportDir(DigitalAeroInfoImportDir->get_text());
	Cfg->SetNationalFlightDBImportDir(NationalFlightDBImportDir->get_text());
	Cfg->SetXPlane900DataImportDir(XPlane900DataImportDir->get_text());
	Cfg->SetGPSSimLon(SimLon->get_value());
	Cfg->SetGPSSimLat(SimLat->get_value());
	Cfg->SetGPSSimSpd(SimSpd->get_value());
	Cfg->SetGPSSimTrk(SimTrk->get_value());
	Cfg->SetGPSSimAlt(SimAlt->get_value());
	Cfg->SetDisableSS(DisableSS->get_active());
	Cfg->SetDisablePM(DisablePM->get_active());
	Cfg->SetShowWelcomeOnStartup(ShowWelcomeOnStartupCheckbox->get_active());
}

void ConfigDialog::GetConfig() {
	/* Not implemented yet
	SerialPortDevice->set_text(Cfg->GetSerialPortDevice());
	SerialPortBaud->???
	*/
	switch (Cfg->GetGPSSource())
	{
		case Config::gpsd:
			gpsd->set_active();
			break;
		case Config::Serial:
			SerialPort->set_active();
			break;
		case Config::Simulated:
			Simulated->set_active();
			break;
	}
	gpsdHost->set_text(Cfg->GetgpsdHost());
	gpsdPort->set_text(Cfg->GetgpsdPort());
	AutoActivateGPS->set_active(Cfg->GetAutoActivateGPS());
	AutoStartgpsd->set_active(Cfg->GetAutoStartgpsd());
	PDFViewer->set_text(Cfg->GetPDFViewer());
	ImageViewer->set_text(Cfg->GetImageViewer());
	SaveConfigOnExit->set_active(Cfg->GetSaveConfigOnExit());
	UserDataDir->set_text(Cfg->GetUserDataDir());
	DataStoreDir->set_text(Cfg->GetDataStoreDir());
	METARSourceURL->set_text(Cfg->GetMETARSourceURL());
	CenterChartOnGPS->set_active(Cfg->GetCenterOnGPS());
	gpsdCmd->set_text(Cfg->GetgpsdCmd());
	AutoSelectChart->set_active(Cfg->GetAutoSelectChart());
	GPSUpdateFrequency->set_value(((double)Cfg->GetGPSUpdateFrequency())/1000.0);
	dTPPImportDir->set_text(Cfg->GetdTPPImportDir());
	SectionalsImportDir->set_text(Cfg->GetSectionalsImportDir());
	DigitalAeroInfoImportDir->set_text(Cfg->GetDigitalAeroInfoImportDir());
	NationalFlightDBImportDir->set_text(Cfg->GetNationalFlightDBImportDir());
	XPlane900DataImportDir->set_text(Cfg->GetXPlane900DataImportDir());
	SimLon->set_value(Cfg->GetGPSSimLon());
	SimLat->set_value(Cfg->GetGPSSimLat());
	SimSpd->set_value(Cfg->GetGPSSimSpd());
	SimTrk->set_value(Cfg->GetGPSSimTrk());
	SimAlt->set_value(Cfg->GetGPSSimAlt());
	DisableSS->set_active(Cfg->GetDisableSS());
	DisablePM->set_active(Cfg->GetDisablePM());
	ShowWelcomeOnStartupCheckbox->set_active(Cfg->GetShowWelcomeOnStartup());
}
