/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <cstring>
#include <sstream>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <libglademm.h>
#include <glibmm/i18n.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/stock.h>
#include "ImportShapefile.h"
#include "Exception.h"
#include "Util.h"

ImportShapefile::ImportShapefile(Config *Cfg, std::string *GladeFilename, Shapefile *ShapefileData)
{
	this->Cfg=Cfg;
	this->GladeFilename=GladeFilename;

	try
	{
		std::string gladefile;

		refXml = Gnome::Glade::Xml::create(*GladeFilename, "ProgressWindow");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog(_("Error reading data file: ")+*GladeFilename+"\n"+_("Program cannot continue."),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
		return;
	}

	refXml->get_widget("ProgressWindow",ProgressWindow);
	refXml->get_widget("SubProgressBar",SubProgressBar);
	refXml->get_widget("ProgressBar",ProgressBar);
	refXml->get_widget("CancelButton",CancelButton);

	ProgressWindow->set_deletable(false);
	CancelButton->signal_clicked().connect(sigc::mem_fun(this,&ImportShapefile::on_CancelButton_clicked));

	CancelFlag=false;
	this->ShapefileData=ShapefileData;
}

ImportShapefile::~ImportShapefile() {
	delete ProgressWindow;
}

int ImportShapefile::Run()
{
	std::string UserDataDir, OutputFileName;
	Shapefile *NewShapefileData;

	UserDataDir=USERDATADIR;
	Util::ShellExpand(&UserDataDir);
	Gtk::FileChooserDialog FileDialog("Browse to the shape file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	FileDialog.set_current_folder(UserDataDir.c_str());

	/* Add filters */
	Gtk::FileFilter FileFilter;
	FileFilter.set_name("ERSI Shapefile [Polygons] (*.shp)");
	FileFilter.add_pattern("*.shp");
	FileDialog.add_filter(FileFilter);

	/* Add response buttons */
	FileDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	FileDialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	if (FileDialog.run()!=Gtk::RESPONSE_OK)
		return false;

	FileDialog.hide();

	NewShapefileData=new Shapefile();

	try {
		NewShapefileData->Load(FileDialog.get_filename());
	} catch (Exception& Ex) {
		Ex.Show();
		delete NewShapefileData;
		return false;
	}

	OutputFileName=(std::string(USERDATADIR)+std::string("/ImportedShapefile"));
	Util::ShellExpand(&OutputFileName);

	/* Omit the extension since .shp and .shx files are saved */
	try {
		NewShapefileData->SavePolyLines(OutputFileName);
	} catch (Exception& Ex) {
		Ex.Show();
	}

/* Adding a progress dialog is a job for another day
	SubProgressBar->set_text("Reading");
	ProgressBar->set_text("Total");
	ProgressWindow->show();

	if (CancelFlag)
		return false;
*/
	delete NewShapefileData;

	return true;
}

void ImportShapefile::on_CancelButton_clicked() {
	CancelFlag=true;
}
