/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifdef HAVE_CONFIG_H
 #include "../config.h"
#endif

#ifndef METARS_H_
#define METARS_H_

#include <list>
#ifdef HAVE_LIBCURL
#include <curl/curl.h>
#endif
#include "METAR.h"
#include "Config.h"

class METARs
{
public:
	METARs(Config *Cfg);
	virtual ~METARs();

	std::string CycleURL(short Offset);
	bool Download();
	bool DownloadCycleFileURL(std::string URL);
	std::list<METAR> *GetStations();
	size_t GetStationCount();
	void CleanUp();
	bool Save();
	bool Load();
	void Clear();

private:
	std::list<METAR> Stations;
	static size_t CurlWriteHandler(void *ptr, size_t size, size_t nmemb, METARs *stream);

#ifdef HAVE_LIBCURL
	void CurlCleanup(CURL *Curl);
#endif

	char *CurlData;
	size_t CurlDataSize;
	Config *Cfg;
};

#endif /*METARS_H_*/
