/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include "RoutePoint.h"

/* A RoutePoint is a Waypoint that's part of a route.  It includes such
 * additional things as a target altitude
 */

RoutePoint::RoutePoint()
{
	Altitude=0;
}

RoutePoint::RoutePoint(Waypoint *WP) {
	this->ID=WP->GetID();
	this->Name=WP->GetName();
	this->Lon=WP->GetLon();
	this->Lat=WP->GetLat();
	this->LocType=WP->GetLocType();
	Altitude=0;
}

void RoutePoint::SetAltitude(unsigned int Altitude)
{
	this->Altitude=Altitude;
}

void RoutePoint::SetLonLatAlt(double Lon, double Lat,unsigned int Alt)
{
	SetLonLat(Lon,Lat);
	SetAltitude(Alt);
}

unsigned int RoutePoint::GetAltitude()
{
	return Altitude;
}

bool RoutePoint::MakeBinary(struct RoutePoint_Binary_struct *BinaryData) {
	/* Sanity checking */
	if (
		Altitude<INT32_MIN || Altitude>INT32_MAX
		) {
		return false;
	}

	BinaryData->Altitude=Altitude;
	if (!Waypoint::MakeBinary(&(BinaryData->Waypoint))) {
		return false;
	}

	return true;
}

void RoutePoint::SetFromBinary(struct RoutePoint_Binary_struct *BinaryData) {
	Altitude=BinaryData->Altitude;
	Waypoint::SetFromBinary(&(BinaryData->Waypoint));
}
